@@ -1,11 +1,33 @@
 package operator
 
 import (
+	"sync"
+
 	"k8s.io/klog"
 
 	operatorv1 "github.com/openshift/api/operator/v1"
 )
 
+type TryOnce struct {
+	lock      sync.Mutex
+	succeeded bool
+}
+
+func (o *TryOnce) Do(f func() error) error {
+	o.lock.Lock()
+	defer o.lock.Unlock()
+
+	if o.succeeded {
+		return nil
+	}
+
+	err := f()
+	o.succeeded = err == nil
+	return err
+}
+
+var once = TryOnce{}
+
 func syncControllers(c serviceCAOperator, operatorConfig *operatorv1.ServiceCA) error {
 	// Any modification of resource we want to trickle down to force deploy all of the controllers.
 	// Sync the controller NS and the other resources. These should be mostly static.
@@ -14,17 +36,19 @@ func syncControllers(c serviceCAOperator, operatorConfig *operatorv1.ServiceCA)
 		return err
 	}
 
-	err = manageSignerControllerResources(c, &needsDeploy)
-	if err != nil {
+	// Remove resources related to 4.3 deployments at most once. These resources don't
+	// constitute an operational concern, so it is not necessarily to monitor for their
+	// presence.
+	//
+	// The 4.3 deployments do have an operational impact, and are continually monitored
+	// for removal via a goroutine in RunOperator.
+	//
+	// This code can be removed in 4.5 since downgrade to 4.3 will no longer be possible.
+	if err := once.Do(func() error { return cleanupDeprecatedResources(c) }); err != nil {
 		return err
 	}
 
-	err = manageAPIServiceControllerResources(c, &needsDeploy)
-	if err != nil {
-		return err
-	}
-
-	err = manageConfigMapCABundleControllerResources(c, &needsDeploy)
+	err = manageControllerResources(c, &needsDeploy)
 	if err != nil {
 		return err
 	}
@@ -40,32 +64,8 @@ func syncControllers(c serviceCAOperator, operatorConfig *operatorv1.ServiceCA)
 		return err
 	}
 
-	// Sync the signing controller.
-	configModified, err := manageSignerControllerConfig(c.corev1Client, c.eventRecorder)
-	if err != nil {
-		return err
-	}
-	_, err = manageSignerControllerDeployment(c.appsv1Client, c.eventRecorder, operatorConfig, needsDeploy || caModified || configModified)
-	if err != nil {
-		return err
-	}
-
-	// Sync the API service controller.
-	configModified, err = manageAPIServiceControllerConfig(c.corev1Client, c.eventRecorder)
-	if err != nil {
-		return err
-	}
-	_, err = manageAPIServiceControllerDeployment(c.appsv1Client, c.eventRecorder, operatorConfig, needsDeploy || caModified || configModified)
-	if err != nil {
-		return err
-	}
-
-	// Sync the API service controller.
-	configModified, err = manageConfigMapCABundleControllerConfig(c.corev1Client, c.eventRecorder)
-	if err != nil {
-		return err
-	}
-	_, err = manageConfigMapCABundleControllerDeployment(c.appsv1Client, c.eventRecorder, operatorConfig, needsDeploy || caModified || configModified)
+	// Sync the controller.
+	_, err = manageDeployment(c.appsv1Client, c.eventRecorder, operatorConfig, needsDeploy || caModified)
 	if err != nil {
 		return err
 	}
